"""
# Dimensionality of the resulting word vectors.
num_features = 300

# Minimum word count threshold.
min_word_count = 8

# Number of threads to run in parallel.
num_workers = multiprocessing.cpu_count()

# Context window length.
context_size = 10

# Downsample setting for frequent words.
downsampling = 1e-3

# Seed for the RNG, to make the results reproducible.
seed = 1

"""
import os
import multiprocessing
import gensim.models.word2vec as w2v
import pandas as pd

class Vectorise:

    def __init__(self, num_features=300, min_word_count=8, 
            context_size=10, downsampling=1e-3, seed=1, model="w2v"):
            self.model_type = model
            if model == "w2v":
                self.model = w2v.Word2Vec(sg=0,  seed=seed, workers=multiprocessing.cpu_count(), size=num_features, min_count=min_word_count, window=context_size, sample=downsampling)

    def add_verbatim(self, sentences):
        if self.model_type == "w2v":
            self.model.build_vocab(sentences)


    def train(self, sentences):
        if self.model_type == "w2v":
            self.model.train(sentences, total_examples=self.model.corpus_count, epochs=self.model.iter)
            self.ordered_vocab = pd.DataFrame([{"word":term, "index":voc.index, "count":voc.count} for term, voc in self.model.wv.vocab.items()])

    def save(self, filename):
        if not os.path.exists("trained"):
            os.makedirs("trained")

        if self.model_type == "w2v":
            self.model.save(os.path.join("trained", filename + ".w2v"))

    def load(self, filename):

        if self.model_type == "w2v":
            self.model = w2v.Word2Vec.load(os.path.join("trained", filename + ".w2v"))




import hdbscan
import pandas as pd


class Cluster():
    def __init__(self, min_cluster_size=15, min_samples=1, alpha=1.0, cluster_selection_method='leaf'):
        self.min_cluster_size = min_cluster_size
        self.min_samples = min_samples
        self.alpha = alpha
        self.labels = []

    def fit(self, vectors):
        self.vectors = vectors
        self.algorithm = hdbscan.HDBSCAN(min_cluster_size=15, min_samples=1, alpha=1.00, cluster_selection_method='leaf').fit(vectors)

        clusters = pd.DataFrame({"label": self.algorithm.labels_}).reset_index()
        counts = clusters.label.value_counts().to_dict()
        good = {k: v for k, v in counts.items() if v < 1000}
        clusters["ok"] = clusters.label.apply(lambda x: x in good and x > -1)
        n_clusters = len(clusters.loc[clusters["ok"], "label"].unique())
        print("FOUND {} CLUSTERS".format(n_clusters))
        self.clusters = clusters



import pandas as pd
import numpy as np
from linguistics.vectorise import Vectorise
from linguistics.io import io


class Utils:

    @staticmethod
    def compare_flare_to_dict(modelname, cutoff=200):
        w2v = Vectorise()
        w2v.load(modelname)
        flare = io.read_file(modelname + "_flare.csv")
        flare['size'].replace('', np.nan, inplace=True)
        flare.dropna(subset=['size'], inplace=True)
        keywords = list(flare.id)
        words = []
        for word, vocab_obj in w2v.model.wv.vocab.items():
            if vocab_obj.count < cutoff:
                continue
            if word in keywords:
                words.append(
                    {"word": word,
                     "count": vocab_obj.count,
                     "in_model": "True"})
            else:
                words.append(
                    {"word": word,
                     "count": vocab_obj.count,
                     "in_model": "False"})

        return pd.DataFrame(words)


    @staticmethod
    def meta_data(modelname, unit="s"):
        w2v = Vectorise()
        try:
            w2v.load(modelname)
            no_of_words = len(w2v.model.wv.vocab.keys())
        except Exception as e:
            print("No vectorise model yet")
            no_of_words = 0

        verbatim = io.read_file(modelname + ".csv")
        verbatim["datetime"] = pd.to_datetime(verbatim.created_utc, unit=unit)

        return {
            "posts": len(verbatim),
            "words": no_of_words,
            "start_date": verbatim.datetime.min(),
            "end_date": verbatim.datetime.max()
        }




import os
from linguistics.pickle import Pickle
from linguistics.dictionary import Dictionary
from tqdm import tqdm
import multiprocessing
from multiprocessing import Process
import glob
import pandas as pd
import numpy as np
from gensim.matutils import corpus2csc
from array_split import array_split, shape_split


class SparkTime:

    def __init__(self):
        self.frequency = "week"
        self.unit = "s"
        self.temp = "temp"
        if not os.path.exists(self.temp):
            os.makedirs(self.temp)
        for the_file in os.listdir(self.temp):
            file_path = os.path.join(self.temp, the_file)
            try:
                if os.path.isfile(file_path):
                    os.unlink(file_path)
            except Exception as e:
                print(e)

    def add_keywords(self, filename):
        flare = pd.read_csv(filename + "_flare.csv")
        flare['size'].replace('', np.nan, inplace=True)
        flare.dropna(subset=['size'], inplace=True)

        self.keywords = list(flare.id)

        self.number_of_cores = multiprocessing.cpu_count()

    def create(self, filename):

        # additional_keywords = ["game_of_thrones"]
        # for word in additional_keywords:
        #     keywords.append(word)

        self.verbatim = pd.read_csv(
            filename + "_spark_sentence.csv").fillna('')
        self.verbatim["datetime"] = pd.to_datetime(
            self.verbatim["date"], unit=self.unit)
        smarter_corpus = Pickle().read(filename)
        self.dct = Dictionary()
        self.dct.read(filename)
        bow_corpus = [self.dct.dct.doc2bow(line) for line in smarter_corpus]
        self.term_doc_mat = corpus2csc(bow_corpus)

        list_of_ids = np.array(
            range(0, len(list(self.dct.dct.token2id.keys()))))

        self.list_of_lists = array_split(list_of_ids, self.number_of_cores)

    def set_frequency(self, freq="day"):
        if freq == "hour":
            self.verbatim['date'] = self.verbatim['datetime'].dt.strftime(
                '%Y-%m-%d-%H')
        elif freq == "day":
            self.verbatim['date'] = self.verbatim['datetime'].dt.strftime(
                '%Y-%m-%d')
        else:
            self.verbatim['date'] = self.verbatim['datetime'].dt.strftime(
                '%Y-%U')

    def do_pivot_init(self, i, df_temp):

        #   if keyword == "year-weeks" :
        #       return

        keyword = self.dct.dct.get(i)
        df_temp[keyword] = self.term_doc_mat.getrow(i).todense().T
        # df_temp = df_temp[(df_temp["yearweeks"]>'2017-01-01')&(df_temp["yearweeks"]<'2017-12-31')]
        pivot = pd.pivot_table(
            df_temp, values=df_temp.columns.values, index='yearweeks', aggfunc='sum')
        df_pivot = pivot.reset_index()
        df_pivot = df_pivot[df_pivot["yearweeks"] != "NaT"]
        df_pivot.to_csv(self.temp + "/" + keyword + '.csv', index=False)

        del df_temp[keyword]

    def sep_proc(self, index_range):
        df_temp = pd.DataFrame()
        df_temp['yearweeks'] = self.verbatim.date
        for i in tqdm(index_range):
            self.do_pivot_init(i, df_temp)

    def run(self):

        procs = []

        for list_of_ids in self.list_of_lists:
            proc = Process(target=self.sep_proc, args=([list_of_ids]))
            procs.append(proc)
            proc.start()

        for proc in procs:
            proc.join()

    def collate(self):
        list_ = []
        for file_ in glob.glob(self.temp + "/*.csv"):
            list_.append(pd.read_csv(file_, index_col='yearweeks'))

        self.frame = pd.concat(list_, axis=1)
        pivot_full_corpus = pd.pivot_table(
            self.verbatim, values="body", index="date", aggfunc="count")
        self.frame["fullCorpus"] = pivot_full_corpus["body"]

    def read(self, filename):
        self.frame = pd.read_csv(filename + '_spark_time.csv', index_col="yearweeks")

    def save(self, filename):
        self.frame.to_csv(filename + '_spark_time.csv')

    def rolling(self, filename, window=5):
        self._rolling = self.frame.rolling(window=window).mean()
        self._rolling = self._rolling.fillna(0)
        self._rolling.to_csv(filename + "_spark_time_avg.csv")

    def normalise(self, spark_time, filename=None):
        for col in spark_time.columns:
            spark_time[col] = spark_time[col]/spark_time["fullCorpus"]

        if filename is not None:
            spark_time.to_csv(filename + "spark_time_norm.csv")
        else:
            return spark_time



import numpy as np
import pandas as pd
from gensim.matutils import corpus2csc
from linguistics.dictionary import Dictionary
from linguistics.pickle import Pickle
from tqdm import tqdm


class Overlap:

    def __init__(self):
        self.dct = ""
        self.term_doct_mat = ""
        self.overlap = ""
        self.norm_overlap = ""
        self.smarter_corpus = ""

    def create(self, filename):

        # additional_keywords = ["game_of_thrones"]
        # for word in additional_keywords:
        #     keywords.append(word)
        self.smarter_corpus = Pickle().read(filename)
        self.dct = Dictionary()
        self.dct.read(filename)
        bow_corpus = [self.dct.dct.doc2bow(set(line))
                      for line in self.smarter_corpus]
        self.term_doc_mat = corpus2csc(bow_corpus)

        term_term_mat = np.dot(self.term_doc_mat, self.term_doc_mat.T)

        self.overlap = pd.DataFrame(term_term_mat.todense())
        new_columns = [self.dct.dct.get(x)
                       for x in self.overlap.columns.values]
        self.overlap.index = new_columns
        self.overlap.columns = new_columns
        self.overlap = self.overlap.reset_index().rename(
            columns={"index": "id"})

    def save(self, filename):
        self.overlap.to_csv(filename + '_overlap.csv', index=False)

    def normalise(self, filename):
        words = self.overlap.set_index("id").columns
        overlap_temp = self.overlap.set_index("id")
        self.norm_overlap = pd.DataFrame()
        for i in tqdm(range(len(words))):
            for j in range(len(words)):
                self.norm_overlap.loc[words[i], words[j]] = overlap_temp.loc[words[i], words[j]]/(
                    self.dct.dct.dfs[self.dct.dct.token2id[words[j]]])
                + overlap_temp.loc[words[j], words[i]] / \
                    (self.dct.dct.dfs[self.dct.dct.token2id[words[i]]])
                -2*self.dct.dct.dfs[self.dct.dct.token2id[words[j]]
                                    ]/len(self.smarter_corpus)

        self.norm_overlap.reset_index().rename(columns={"index": "id"}).to_csv(
            filename + '_overlap_new.csv', index=False)



from sklearn.decomposition import PCA, FastICA


class Decompose():

    def __init__(self, n_components=15, copy=True, whiten=True, mode="pca", max_iter=2000, model=None):
        if model is not None:
            self.model = model
        else:
            if mode == "pca":
                self.model = PCA(n_components=n_components, copy=copy, whiten=whiten)
            elif mode == "ica":
                self.model = FastICA(n_components=n_components, whiten=whiten, max_iter=max_iter)

    def fit(self, X):
        self.model.fit(X)

    def transform(self, X):
        return self.model.transform(X)

    def fit_transform(self, X):
        data = self.model.fit_transform(X)
        return data




import setuptools

setuptools.setup(
    name='ads_linguistics',
    version='2.0.9',
    description='The ADS Linguistic Library',
    url='https://flamingo-james-b@bitbucket.org/flamingodf/linguistics.git', author='Applied Data Science Team',
    author_email='james.burnett@flamingogroup.com',
    scripts=['bin/run_clusters', 'bin/create_all_files', 'bin/create_all_files_wo_dict', 'bin/compare_flare_to_dict', 'bin/get_meta',
             'bin/update_flare_counts'],
    license='Flamingo',
    packages=setuptools.find_packages(),
    install_requires=['array-split', 'pandas', 'scikit-learn', 'hdbscan', 'gensim',  'nltk',  'scipy',
                      'six', 'tqdm'],
    zip_safe=False)

# import nltk
# nltk.download('averaged_perceptron_tagger')
# nltk.download('wordnet')



import nltk
import gensim
from gensim.models import Phrases
import pandas as pd
from nltk.stem import WordNetLemmatizer
from nltk.corpus import stopwords, wordnet
from gensim.models.phrases import Phraser
from gensim.corpora import Dictionary
from tqdm import tqdm
import linguistics.gensim_utils as gensim_utils


tqdm.pandas()


class Preprocess:
    def __init__(self, data, model, lang, keep_words=[]):
        self.data = data
        self.lang = lang
        self.model = model
        self.keep_words = keep_words
        self._stopwords = stopwords.words(lang)
        self.new_data = pd.DataFrame()
        self.tokenizer = self.load_nltk_tokenizer()
        self.lemmatizer = WordNetLemmatizer()
        self.sentences = []
        self.tokenized_body = []

    def load_nltk_tokenizer(self):
        file_path = "tokenizers/punkt/{}.pickle".format(self.lang)
        try:
            tokenizer = nltk.data.load(file_path)
            return tokenizer
        except Exception as e:
            print("No tokenizer found for language = {}".format(self.lang))

    def set_frequency(self, freq="day"):
        if freq == "hour":
            self.new_data['datestring'] = self.new_data['datetime'].dt.strftime(
                '%Y-%m-%d-%H')
        elif freq == "day":
            self.new_data['datestring'] = self.new_data['datetime'].dt.strftime(
                '%Y-%m-%d')
        else:
            self.new_data['datestring'] = self.new_data['datetime'].dt.strftime(
                '%Y-%U')

    def create_sentences_dataframe(self):
        self.tokenized_body = self.data['body'].progress_apply(
            lambda x: self.tokenizer.tokenize(str(x).lower()))
        print("finished splitting")
        rows = []
        for i, row in tqdm(self.data.iterrows()):
            for a in self.tokenized_body[i]:
                row.sentence = a
                if "comment_id" in row.keys():
                    rows.append({"sentence": '"' + a + '"',
                                 "created_utc": row["created_utc"],
                                 "comment_id": row["comment_id"]})
                else:
                    rows.append({"sentence": '"' + a + '"',
                                 "created_utc": row["created_utc"]})
        self.new_data = pd.DataFrame(rows).fillna("")
        self.new_data["datetime"] = pd.to_datetime(self.new_data["created_utc"], unit="s")
        print(len(self.new_data))

    # Cleans and tokenizes each sentence in list (Lower cases words, removes stopwords/numbers etc.) sentences
    # is a list of lists containing cleaned words from each sentence in CSV.

    def clean_sentences(self, lemma=True):
        df = self.new_data
        sentences = []
        for sentence in tqdm(df.sentence.values):
            if len(sentence) > 0:
                pre_process = gensim_utils.simple_preprocess(
                    sentence, deacc=True)
                important_words = []
                for word in pre_process:
                    if word not in self._stopwords:
                        if lemma and word not in self.keep_words:
                            lemmatized_word = self.lemmatize_word(word)
                            important_words.append(lemmatized_word)
                        else:
                            important_words.append(word)
                sentences.append(important_words)
            else:
                sentences.append([])

        self.sentences = sentences

    def gram_it(self, min_count=5):
        phrases = Phrases(self.sentences, min_count=min_count)  # Train a model.
        bigram = Phraser(phrases)  # Exports the model.
        trigram = Phrases(bigram[self.sentences], min_count=min_count)  # Another training which yields any trigrams.
        tokenized_sentences = list((trigram[bigram[self.sentences]]))
        return tokenized_sentences

    def lemmatize_word(self, word):
        tag = self.get_wordnet_pos(word)
        lemmatized_word = self.lemmatizer.lemmatize(word, tag)
        return lemmatized_word

    @staticmethod
    def get_wordnet_pos(word):
        """Find a word's Part of Speech tag and map it a wordnet accepted form.
        https://www.ling.upenn.edu/courses/Fall_2003/ling001/penn_treebank_pos.html
        """
        tag = nltk.pos_tag([word])[0][1][0]
        tag_dict = {
            "J": wordnet.ADJ,
            "N": wordnet.NOUN,
            "V": wordnet.VERB,
            "R": wordnet.ADV
        }

        tag = tag_dict.get(tag, wordnet.NOUN)  # Presume a word is a noun if it isn't in the above dict.
        return tag



from gensim.corpora import Dictionary as Dict
from linguistics.pickle import Pickle
import os
from tqdm import tqdm
import pandas as pd
import numpy as np


class Dictionary():

    def __init__(self, corpus=None):
        if corpus is None:
            self.dct = None
        self.dct = Dict(corpus)
        self.keywords = []
        # bow_corpus = [dct.doc2bow(line) for line in smarter_corpus]
        # term_doc_mat = corpus2csc(bow_corpus)

        # saving

    def save(self, filename):
        if not os.path.exists("trained"):
            os.makedirs("trained")
        self.dct.save_as_text("trained/" + filename + ".DictSaved")

    def read(self, filename):
        self.dct = Dict.load_from_text("trained/" + filename + ".DictSaved")

    def reduce(self, filename):
        flare = pd.read_csv(filename + "_flare.csv")
        flare['size'].replace('', np.nan, inplace=True)
        flare.dropna(subset=['size'], inplace=True)

        self.keywords = list(flare.id)

        self.read(filename + "_full")

        corpus = Pickle().read(filename + "_full")

        smaller_corpus = []
        for sentence in tqdm(corpus):
            cleanups = [token.lower() for token in sentence if token in self.keywords]
            smaller_corpus.append(cleanups)

        self.dct = Dict(smaller_corpus)

        Pickle().save(smaller_corpus, filename)

        self.save(filename)



import pickle
import os

class Pickle():

    @staticmethod
    def save(corpus, filename):
        if not os.path.exists("trained"):
            os.makedirs("trained")
        with open("trained/" + filename + '.tokenizer.pickle', 'wb') as handle:
            pickle.dump(corpus, handle, protocol=pickle.HIGHEST_PROTOCOL)

    @staticmethod
    def read(filename):
        with open("trained/" + filename + '.tokenizer.pickle', 'rb') as handle:
            corpus = pickle.load(handle)
        return corpus 


import pandas as pd


class io:

    @staticmethod
    def read_file(filename, file="csv"):
        if file == "csv":
            return pd.read_csv(filename)

    @staticmethod
    def save_file(data, filename, file="csv", index=False):
        if file == "csv":
            data.to_csv(filename, index=index)

    @staticmethod
    def check_args(sys, no_of_args=2, message="I need a) filename"):
        if len(sys.argv) < no_of_args:
            print(message)
            sys.exit()

        filename = sys.argv[1]
        modelname = filename.split('/')[-1]
        modelname = modelname[:-4]

        return filename, modelname
